// SleepTune - pauses iTunes after a user-defined period of time.
// Copyright (C) 2005  Martey Dodoo
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using System.ComponentModel;
using System.Diagnostics;
using iTunesLib;

namespace SleepTune
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.NumericUpDown numTime;
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Label lblMinutes;
		private System.Windows.Forms.MenuItem mnuModify;
		private System.Timers.Timer timer;
		private System.Windows.Forms.MenuItem mnuCancel;
		private System.Windows.Forms.Button btnAbout;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Label lblMakeiTunesSleepIn;
		private System.Windows.Forms.ContextMenu contextMenu;
		
		private NotifyIcon trayIcon;
		private iTunesLib.IiTunes iTunesApp;
		
		public MainForm()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			trayIcon = new NotifyIcon();
			trayIcon.Text = Application.ProductName;
			trayIcon.Visible = true;
			System.IO.Stream icoStream = this.GetType().Assembly.GetManifestResourceStream("sound.ico");
			trayIcon.Icon = new Icon(icoStream);
			trayIcon.ContextMenu = contextMenu;
		}
		
		[STAThread]
		public static void Main(string[] args)
		{
			#region mutex creation
			/// http://www.codeproject.com/csharp/SingleInstanceApplication.asp
			bool mutexCreated = false;
			// The name of the mutex is to be prefixed with Local\ to make
			// sure that its is created in the per-session namespace,
			// not in the global namespace.
			string mutexName = "Local\\" + 
			  System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;
			Mutex mutex = null;
			try{
				mutex = new Mutex(false, mutexName, out mutexCreated);
			}catch(Exception ex){
				MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace + "\n\n" + 
				  "Could not create mutex! Application must now exit.",
				  "Mutex Creation Error",MessageBoxButtons.OK,
				  MessageBoxIcon.Error);
				Application.Exit();
			}
			#endregion
			if(mutexCreated && mutex.WaitOne()){
				Application.Run(new MainForm());
				#region mutex destruction
				GC.KeepAlive(mutex);
				try{
					mutex.ReleaseMutex();
				}catch(ApplicationException ex){
					MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace+"\n\n" +
					  "Could not release mutex! Application will continue to " +
					  "exit.","Mutex Release Error",MessageBoxButtons.OK,
					  MessageBoxIcon.Error);
					GC.Collect();
				}
				#endregion
			}else{
				MessageBox.Show("SleepTune is already running.","SleepTune",
				  MessageBoxButtons.OK,MessageBoxIcon.Information);
			}
			mutex.Close();
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.contextMenu = new System.Windows.Forms.ContextMenu();
			this.lblMakeiTunesSleepIn = new System.Windows.Forms.Label();
			this.btnCancel = new System.Windows.Forms.Button();
			this.btnAbout = new System.Windows.Forms.Button();
			this.mnuCancel = new System.Windows.Forms.MenuItem();
			this.timer = new System.Timers.Timer();
			this.mnuModify = new System.Windows.Forms.MenuItem();
			this.lblMinutes = new System.Windows.Forms.Label();
			this.btnOK = new System.Windows.Forms.Button();
			this.numTime = new System.Windows.Forms.NumericUpDown();
			((System.ComponentModel.ISupportInitialize)(this.timer)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numTime)).BeginInit();
			this.SuspendLayout();
			// 
			// contextMenu
			// 
			this.contextMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.mnuModify,
						this.mnuCancel});
			// 
			// lblMakeiTunesSleepIn
			// 
			this.lblMakeiTunesSleepIn.Location = new System.Drawing.Point(16, 16);
			this.lblMakeiTunesSleepIn.Name = "lblMakeiTunesSleepIn";
			this.lblMakeiTunesSleepIn.Size = new System.Drawing.Size(120, 16);
			this.lblMakeiTunesSleepIn.TabIndex = 3;
			this.lblMakeiTunesSleepIn.Text = "Make iTunes sleep in";
			// 
			// btnCancel
			// 
			this.btnCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.Image = ((System.Drawing.Image)(resources.GetObject("btnCancel.Image")));
			this.btnCancel.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnCancel.Location = new System.Drawing.Point(112, 48);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(88, 32);
			this.btnCancel.TabIndex = 1;
			this.btnCancel.Text = "&Cancel";
			this.btnCancel.Click += new System.EventHandler(this.BtnCancelClick);
			// 
			// btnAbout
			// 
			this.btnAbout.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnAbout.Image = ((System.Drawing.Image)(resources.GetObject("btnAbout.Image")));
			this.btnAbout.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnAbout.Location = new System.Drawing.Point(216, 48);
			this.btnAbout.Name = "btnAbout";
			this.btnAbout.Size = new System.Drawing.Size(88, 32);
			this.btnAbout.TabIndex = 2;
			this.btnAbout.Text = "&About";
			this.btnAbout.Click += new System.EventHandler(this.BtnAboutClick);
			// 
			// mnuCancel
			// 
			this.mnuCancel.Index = 1;
			this.mnuCancel.Text = "Cancel";
			this.mnuCancel.Click += new System.EventHandler(this.BtnCancelClick);
			// 
			// timer
			// 
			this.timer.Interval = 60000;
			this.timer.SynchronizingObject = this;
			this.timer.Elapsed += new System.Timers.ElapsedEventHandler(this.TimerElapsed);
			// 
			// mnuModify
			// 
			this.mnuModify.Index = 0;
			this.mnuModify.Text = "Modify";
			this.mnuModify.Click += new System.EventHandler(this.mnuModifyClick);
			// 
			// lblMinutes
			// 
			this.lblMinutes.Location = new System.Drawing.Point(200, 16);
			this.lblMinutes.Name = "lblMinutes";
			this.lblMinutes.Size = new System.Drawing.Size(48, 16);
			this.lblMinutes.TabIndex = 5;
			this.lblMinutes.Text = "minutes";
			// 
			// btnOK
			// 
			this.btnOK.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.Image = ((System.Drawing.Image)(resources.GetObject("btnOK.Image")));
			this.btnOK.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnOK.Location = new System.Drawing.Point(8, 48);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(88, 32);
			this.btnOK.TabIndex = 0;
			this.btnOK.Text = "&OK";
			this.btnOK.Click += new System.EventHandler(this.BtnOKClick);
			// 
			// numTime
			// 
			this.numTime.Location = new System.Drawing.Point(144, 16);
			this.numTime.Maximum = new System.Decimal(new int[] {
						360,
						0,
						0,
						0});
			this.numTime.Name = "numTime";
			this.numTime.Size = new System.Drawing.Size(48, 20);
			this.numTime.TabIndex = 4;
			this.numTime.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			this.numTime.Value = new System.Decimal(new int[] {
						30,
						0,
						0,
						0});
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(312, 85);
			this.Controls.Add(this.lblMinutes);
			this.Controls.Add(this.numTime);
			this.Controls.Add(this.lblMakeiTunesSleepIn);
			this.Controls.Add(this.btnAbout);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOK);
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "MainForm";
			this.ShowInTaskbar = false;
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.Text = "SleepTune";
			this.TopMost = true;
			((System.ComponentModel.ISupportInitialize)(this.timer)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numTime)).EndInit();
			this.ResumeLayout(false);
		}
		#endregion
		
		/// <summary>
		/// This is supposed to remove trayIcon when the application closes.
		/// </summary>
		protected override void Dispose(bool disposing){
			if(disposing){
				this.trayIcon.Dispose();
			}
			base.Dispose(disposing);
		}
		
		void BtnOKClick(object sender, System.EventArgs e)
		{
			timer.Enabled = true;
			trayIcon.Text = Application.ProductName + ": " + 
			  numTime.Value.ToString() + " minutes";
			this.Hide();
		}
		
		void BtnCancelClick(object sender, System.EventArgs e)
		{
			Close();
		}
		
		void BtnAboutClick(object sender, System.EventArgs e)
		{
			Process pBrowser = new Process();
			pBrowser.StartInfo.FileName = "http://www.marteydodoo.com/";
			pBrowser.Start();
		}
		
		void TimerElapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			switch ((int)numTime.Value){
			case 0:
				timer.Enabled = false;
				try{
					iTunesApp = new iTunesAppClass();;
				}catch(Exception ex){
					MessageBox.Show(ex.Message,"Could not find iTunes!",
					                MessageBoxButtons.OK,MessageBoxIcon.Error);
					Close();
				}
				// pause iTunes
				iTunesApp.Pause();
				Close();
				break;
			case 1:
				numTime.Value = numTime.Value - 1;
				trayIcon.Text = Application.ProductName + 
				  ": Less than a minute";
				break;
			case 2:
				numTime.Value = numTime.Value - 1;
				trayIcon.Text = Application.ProductName + ": " +
					numTime.Value.ToString() + " minute";
				break;
			default:
				numTime.Value = numTime.Value - 1;
				trayIcon.Text = Application.ProductName + ": " +
					numTime.Value.ToString() + " minutes";
				break;
			}
		}
		
		void mnuModifyClick(object sender, System.EventArgs e)
		{
			this.Show();
		}
		
	}
}
